// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CONTEXT_H
#define OB_CONTEXT_H

namespace CORBA
{

class Context;
typedef Context* Context_ptr;
typedef Context_ptr ContextRef;

} // End of namespace CORBA

void OBDuplicate(CORBA::Context_ptr);
void OBRelease(CORBA::Context_ptr);

namespace CORBA
{

//
// The Context class
//
typedef OB::ObjVar< Context > Context_var;
typedef OB::ObjOut< Context > Context_out;


class Context : public OB::RefCount
{
    String_var name_;
    Context_var parent_;
    OB::StringSeq_var values_;

    //
    // Hide copy-constructor and assignment operator
    //
    Context(const Context&);
    void operator=(const Context&);

protected:

    Context(const char*);
    Context(const char*, Context_ptr);
    friend class ORB; // ORBs may create Contexts
    
public:

    //
    // Standard IDL to C++ Mapping
    //
    static inline Context_ptr _duplicate(Context_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Context_ptr _nil()
    { return 0; }

    const char* context_name() const { return name_; }
    Context_ptr parent() const { return parent_; }

    void create_child(const char*, Context_out);

    void set_one_value(const char*, const Any&);
    void set_values(NVList_ptr);
    void delete_values(const char*);
    void get_values(const char*, Flags, const char*, NVList_out);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_getValues(const char*, Flags, const char*, OB::StringSeq&);
    static Context* _OB_create(OB::StringSeq*);
};

inline void
release(Context_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(Context_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
